function [x, flog, pointlog] = conjgrad(x, f, gradf, options, varargin)

%CONJGRAD Conjugate gradients optimization.
%
%	Description
%	[X, FLOG, POINTLOG] = CONJGRAD(F, X, OPTIONS, GRADF) uses a
%	conjugate gradients algorithm to find the minimum of the function
%	F(X) whose gradient is given by GRADF(X).  Here X is a row vector and
%	F returns a scalar value.  The point at which F has a local minimum
%	is returned as X. A log of the function values after each cycle is
%	(optionally) returned in FLOG, and a log of the points visited is
%	(optionally) returned in POINTLOG.
%
%	CONJGRAD(F, X, OPTIONS, GRADF, P1, P2, ...) allows  additional
%	arguments to be passed to F() and GRADF().
%
%	The optional parameters have the following interpretations.
%
%	verbosemode is set to 1 to verbosemode error values; also logs error
%	values in the return argument ERRLOG, and the points visited in the
%	return argument POINTSLOG.  If verbosemode is set to 0, then only
%	warning messages are verbosemodeed.  If verbosemode is -1, then nothing is
%	verbosemodeed.
%
%	breakratio is a measure of the precision required of the objective
%	function at the solution.  If the absolute difference between the
%	objective function values between two successive steps is less than
%	breakratio times its initial value, then this condition is satisfied.
%   default 1e-6
%
%	maxiter is the maximum number of iterations; default 500.
%
%	lineprec is the precision in parameter space of the line search;
%	default 1E-4.

%  Set up the options.
maxiter = 500;
breakratio = 1e-6;
lineprec = 1e-4;
verbosemode = 0;

loadstruct('options')

% Set up options for line search
line_options = foptions;
% Need a precise line search for success
if lineprec > 0
  line_options(2) = lineprec;
else
  line_options(2) = 1e-4;
end

% Next two lines allow conjgrad to work with expression strings
f = fcnchk(f, length(varargin));
gradf = fcnchk(gradf, length(varargin));

nparams = length(x(:));
fnew = feval(f, x, varargin{:});
flog = fnew;
finit = fnew;
xmaxinit = max(abs(x(:)));
gradnew = feval(gradf, x, varargin{:});
d = -gradnew;		% Initial search direction
br_min = 0;
br_max = 1.0;	% Initial value for maximum distance to search along
tol = sqrt(eps);

j = 1;
if nargout >= 3
  flog(j, :) = fnew;
  if nargout == 4
    pointlog(j, :) = x(:)';
  end
end

while (j <= maxiter)

  xold = x;
  fold = fnew;
  gradold = gradnew;

  gg = gradold(:)'*gradold(:);
  if (gg == 0.0)
    % If the gradient is zero then we are done.
    return;
  end

  % This shouldn't occur, but rest of code depends on d being downhill
  if (gradnew(:)'*d(:) > 0)
    d = -d;
    if verbosemode >= 0
      warning('search direction uphill in conjgrad');
    end
  end

  line_sd = d./norm(d(:));
  [lmin, line_options] = feval('linemin', f, xold, line_sd, fold, ...
    line_options, varargin{:});
  % Set x and fnew to be the actual search point we have found
  x = xold + lmin * line_sd;
  fnew = line_options(8);

  % Check for termination
  if max(abs(fnew - fold)) < breakratio*finit
    return;
  end

  gradnew = feval(gradf, x, varargin{:});

  % Use Polak-Ribiere formula to update search direction
  gamma = ((gradnew(:) - gradold(:))'*(gradnew(:)))/gg;
  d = (d .* gamma) - gradnew;

  if (verbosemode > 0)
    fprintf(1, 'Cycle %4d  Function %11.6f\n', j, line_options(8));
  end

  j = j + 1;
  if nargout >= 2
    flog(j, :) = fnew;
    if nargout == 3
      pointlog(j, :) = x(:)';
    end
  end
end

% If we get here, then we haven't terminated in the given number of 
% iterations.

if (verbosemode >= 0)
  disp(maxitmess);
end
